# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
from django.conf.urls import url, include

from rest_framework import routers

from . import views


router = routers.DefaultRouter()
router.register('notes', views.NoteViewSet)
router.register('releases', views.ReleaseViewSet)

urlpatterns = [
    url(r'^', include(router.urls)),
    url(r'^releases/(?P<pk>\d+)/notes/$', views.NestedNoteView.as_view()),
    url(r'^auth_token/$', views.auth_token),
    url(r'^sync/?$', views.rnasync),
]



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from rest_framework.filters import DjangoFilterBackend
import django_filters

from . import fields, models


class ISO8601DateTimeFilter(django_filters.DateTimeFilter):
    field_class = fields.ISO8601DateTimeField


class TimestampedFilterBackend(DjangoFilterBackend):
    def get_filter_class(self, view, queryset=None):
        filter_class = getattr(view, 'filter_class', None)
        filter_fields = getattr(view, 'filter_fields', None)
        filter_fields_exclude = getattr(view, 'filter_fields_exclude', ())

        if filter_class or filter_fields:
            return super(TimestampedFilterBackend, self).get_filter_class(
                view, queryset=queryset)

        elif queryset and hasattr(queryset, 'model') and issubclass(
                queryset.model, models.TimeStampedModel):
            class AutoFilterSet(self.default_filter_set):
                created_before = ISO8601DateTimeFilter(
                    name='created', lookup_type='lt')
                created_after = ISO8601DateTimeFilter(
                    name='created', lookup_type='gte')

                modified_before = ISO8601DateTimeFilter(
                    name='modified', lookup_type='lt')
                modified_after = ISO8601DateTimeFilter(
                    name='modified', lookup_type='gte')

                class Meta:
                    model = queryset.model
                    fields = ['created_before', 'created_after',
                              'modified_before', 'modified_after']
                    fields.extend(f.name for f in model._meta.fields
                                  if f.name not in ('created', 'modified'))
                    fields = [f for f in fields
                              if f not in filter_fields_exclude]
                    order_by = True
            return AutoFilterSet



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import json

from django.http import HttpResponse, HttpResponseForbidden
from django.shortcuts import get_object_or_404
from django.views.decorators.http import last_modified

from rest_framework import generics
from rest_framework.authtoken.models import Token
from rest_framework.viewsets import ModelViewSet
from synctool.routing import Route

from . import models, serializers
from .utils import get_last_modified_date


rnasync = Route(api_token=None).app('rna', 'rna')
rnasync = last_modified(get_last_modified_date)(rnasync)


def auth_token(request):
    if request.user.is_active and request.user.is_staff:
        token, created = Token.objects.get_or_create(user=request.user)
        return HttpResponse(
            content=json.dumps({'token': token.key}),
            content_type='application/json')
    else:
        return HttpResponseForbidden()


class NoteViewSet(ModelViewSet):
    queryset = models.Note.objects.all()
    serializer_class = serializers.NoteSerializer


class ReleaseViewSet(ModelViewSet):
    queryset = models.Release.objects.all()
    serializer_class = serializers.ReleaseSerializer


class NestedNoteView(generics.ListAPIView):
    model = models.Note
    serializer_class = serializers.NoteSerializer

    def get_queryset(self):
        release = get_object_or_404(models.Release, pk=self.kwargs.get('pk'))
        return release.note_set.all()



METHOD_OVERRIDE_HEADER = 'HTTP_X_HTTP_METHOD_OVERRIDE'


class PatchOverrideMiddleware(object):
    def process_view(self, request, callback, callback_args, callback_kwargs):
        if request.method == 'POST' and request.META.get(METHOD_OVERRIDE_HEADER) == 'PATCH':
            request.method = 'PATCH'



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from django import forms
from django.contrib import admin
from django.utils.timezone import now
from pagedown.widgets import AdminPagedownWidget

from . import models


class NoteAdminForm(forms.ModelForm):
    note = forms.CharField(widget=AdminPagedownWidget())

    class Meta:
        model = models.Note
        fields = '__all__'


class NoteAdmin(admin.ModelAdmin):
    form = NoteAdminForm
    filter_horizontal = ['releases']
    list_display = ('id', 'bug', 'tag', 'note', 'created')
    list_display_links = ('id',)
    list_filter = ('tag', 'is_known_issue', 'releases__product',
                   'releases__version')
    search_fields = ('bug', 'note', 'releases__version')


class ReleaseAdminForm(forms.ModelForm):
    system_requirements = forms.CharField(widget=AdminPagedownWidget(),
                                          required=False)
    text = forms.CharField(widget=AdminPagedownWidget(), required=False)
    release_date = forms.DateTimeField(widget=admin.widgets.AdminDateWidget)

    class Meta:
        model = models.Release
        fields = '__all__'


class ReleaseAdmin(admin.ModelAdmin):
    actions = ['copy_releases', 'set_to_public']
    form = ReleaseAdminForm
    list_display = ('version', 'product', 'channel', 'is_public',
                    'release_date', 'text', 'url')
    list_filter = ('product', 'channel', 'is_public')
    ordering = ('-release_date',)
    search_fields = ('version', 'text')

    def url(self, obj):
        base_url_staging = "https://www-dev.allizom.org/en-US"
        base_url_prod = "https://www.mozilla.com/en-US"
        product = ""

        if obj.product == "Firefox for Android":
            product = "firefox/android"
        if obj.product == "Firefox for iOS":
            product = "firefox/ios"
        elif obj.product == "Firefox" or obj.product == "Firefox Extended Support Release":
            product = "firefox"
        elif obj.product == "Thunderbird":
            product = "thunderbird"
        elif obj.product == "Firefox OS":
            # Special case for Firefox OS. URL are different
            return ('<a href="{staging}/firefox/os/notes/{version}/">Staging</a> / '
                    '<a href="{prod}/firefox/os/notes/{version}/">Public</a>'.format(
                        staging=base_url_staging, product=product, version=obj.version, prod=base_url_prod))

        return ('<a href="{staging}/{product}/{version}/releasenotes/">Staging</a> / '
                '<a href="{prod}/{product}/{version}/releasenotes/">Public</a>'.format(
                    staging=base_url_staging, product=product, version=obj.version, prod=base_url_prod))

    url.allow_tags = True

    def copy_releases(self, request, queryset):
        release_count = 0
        for release in queryset:
            release_count += 1
            copy_count = self.model.objects.filter(
                version__endswith=release.version,
                product=release.product).count()
            notes = list(release.note_set.all())
            copy = release
            copy.id = None
            if copy_count > 1:
                copy.version = 'copy%s-%s' % (copy_count, copy.version)
            else:
                copy.version = 'copy-' + copy.version
            # By default, set it to public. Usually, the copy feature is used
            # when copying aurora => beta or beta => release. We want to review
            # it before going live
            copy.is_public = False
            copy.save()
            copy.note_set.add(*notes)
            copy.note_set.update(modified=now())
        if release_count == 1:
            self.message_user(request, 'Copied Release')
        else:
            self.message_user(request, 'Copied %s Releases' % release_count)

    def set_to_public(self, request, queryset):
        """ Set one or several releases to public """
        queryset.update(is_public=True, modified=now())

admin.site.register(models.Note, NoteAdmin)
admin.site.register(models.Release, ReleaseAdmin)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from rest_framework import serializers

from .models import Note, Release


class HyperlinkedModelSerializerWithPkField(serializers.HyperlinkedModelSerializer):
    def get_default_field_names(self, declared_fields, model_info):
        fields = super(HyperlinkedModelSerializerWithPkField, self).get_default_field_names(
            declared_fields, model_info)
        fields.append('id')
        return fields


class NoteSerializer(HyperlinkedModelSerializerWithPkField):
    class Meta:
        model = Note


class ReleaseSerializer(HyperlinkedModelSerializerWithPkField):
    class Meta:
        model = Release



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from django.conf import settings
from django.db import models
from django.utils.timezone import now
from django_extensions.db.fields import CreationDateTimeField


class TimeStampedModel(models.Model):
    """
    Replacement for django_extensions.db.models.TimeStampedModel
    that updates the modified timestamp by default, but allows
    that behavior to be overridden by passing a modified=False
    parameter to the save method
    """
    created = CreationDateTimeField()
    modified = models.DateTimeField(editable=False, blank=True, db_index=True)

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        if kwargs.pop('modified', True):
            self.modified = now()
        super(TimeStampedModel, self).save(*args, **kwargs)


class Release(TimeStampedModel):
    CHANNELS = ('Nightly', 'Aurora', 'Beta', 'Release', 'ESR')
    PRODUCTS = ('Firefox', 'Firefox for Android',
                'Firefox Extended Support Release', 'Firefox OS',
                'Thunderbird', 'Firefox for iOS')

    product = models.CharField(max_length=255,
                               choices=[(p, p) for p in PRODUCTS])
    channel = models.CharField(max_length=255,
                               choices=[(c, c) for c in CHANNELS])
    version = models.CharField(max_length=255)
    release_date = models.DateTimeField()
    text = models.TextField(blank=True)
    is_public = models.BooleanField(default=False)
    bug_list = models.TextField(blank=True)
    bug_search_url = models.CharField(max_length=2000, blank=True)
    system_requirements = models.TextField(blank=True)

    def major_version(self):
        return self.version.split('.', 1)[0]

    def get_bug_search_url(self):
        if self.bug_search_url:
            return self.bug_search_url

        if self.product == 'Thunderbird':
            return (
                'https://bugzilla.mozilla.org/buglist.cgi?'
                'classification=Client%20Software&query_format=advanced&'
                'bug_status=RESOLVED&bug_status=VERIFIED&bug_status=CLOSED&'
                'target_milestone=Thunderbird%20{version}.0&product=Thunderbird'
                '&resolution=FIXED'
            ).format(version=self.major_version())

        return (
            'https://bugzilla.mozilla.org/buglist.cgi?'
            'j_top=OR&f1=target_milestone&o3=equals&v3=Firefox%20{version}&'
            'o1=equals&resolution=FIXED&o2=anyexact&query_format=advanced&'
            'f3=target_milestone&f2=cf_status_firefox{version}&'
            'bug_status=RESOLVED&bug_status=VERIFIED&bug_status=CLOSED&'
            'v1=mozilla{version}&v2=fixed%2Cverified&limit=0'
        ).format(version=self.major_version())

    def equivalent_release_for_product(self, product):
        """
        Returns the release for a specified product with the same
        channel and major version with the highest minor version,
        or None if no such releases exist
        """
        releases = self._default_manager.filter(
            version__startswith=self.major_version() + '.',
            channel=self.channel, product=product).order_by('-version')
        if not getattr(settings, 'DEV', False):
            releases = releases.filter(is_public=True)
        if releases:
            return sorted(
                sorted(releases, reverse=True,
                       key=lambda r: len(r.version.split('.'))),
                reverse=True, key=lambda r: r.version.split('.')[1])[0]

    def equivalent_android_release(self):
        if self.product == 'Firefox':
            return self.equivalent_release_for_product('Firefox for Android')

    def equivalent_desktop_release(self):
        if self.product == 'Firefox for Android':
            return self.equivalent_release_for_product('Firefox')

    def notes(self, public_only=False):
        """
        Retrieve a list of Note instances that should be shown for this
        release, grouped as either new features or known issues, and sorted
        first by sort_num highest to lowest, which is applied to both groups,
        and then for new features we also sort by tag in the order specified
        by Note.TAGS, with untagged notes coming first, then finally moving
        any note with the fixed tag that starts with the release version to
        the top, for what we call "dot fixes".
        """
        tag_index = dict((tag, i) for i, tag in enumerate(Note.TAGS))
        notes = self.note_set.order_by('-sort_num')
        if public_only:
            notes = notes.filter(is_public=True)
        known_issues = [n for n in notes if n.is_known_issue_for(self)]
        new_features = sorted(
            sorted(
                (n for n in notes if not n.is_known_issue_for(self)),
                key=lambda note: tag_index.get(note.tag, 0)),
            key=lambda n: n.tag == 'Fixed' and n.note.startswith(self.version),
            reverse=True)

        return new_features, known_issues

    def __unicode__(self):
        return '{product} {version} {channel}'.format(
            product=self.product, version=self.version, channel=self.channel)

    class Meta:
        # TODO: see if this has a significant performance impact
        ordering = ('product', '-version', 'channel')
        unique_together = (('product', 'version'),)
        get_latest_by = 'modified'


class Note(TimeStampedModel):
    TAGS = ('New', 'Changed', 'HTML5', 'Feature', 'Language', 'Developer',
            'Fixed')

    bug = models.IntegerField(null=True, blank=True)
    note = models.TextField(blank=True)
    releases = models.ManyToManyField(Release, blank=True)
    is_known_issue = models.BooleanField(default=False)
    fixed_in_release = models.ForeignKey(Release, null=True, blank=True,
                                         related_name='fixed_note_set')
    tag = models.CharField(max_length=255, blank=True,
                           choices=[(t, t) for t in TAGS])
    sort_num = models.IntegerField(default=0)
    is_public = models.BooleanField(default=True)

    def is_known_issue_for(self, release):
        return self.is_known_issue and self.fixed_in_release != release

    def __unicode__(self):
        return self.note

    class Meta:
        get_latest_by = 'modified'



from django.core.exceptions import ObjectDoesNotExist

from .models import Note, Release


def get_last_modified_date(*args, **kwargs):
    """Returns the date of the last modified Note or Release.

    For use with Django's last_modified decorator.
    """
    try:
        latest_note = Note.objects.latest()
        latest_release = Release.objects.latest()
    except ObjectDoesNotExist:
        return None

    return max(latest_note.modified, latest_release.modified)


def migrate_versions():
    for r in Release.objects.filter(version__endswith='.0.0').only(
            'channel', 'version'):
        if r.channel == 'Release':
            Release.objects.filter(id=r.id).update(version=r.version[:-2])
        elif r.channel == 'Aurora':
            Release.objects.filter(id=r.id).update(version=r.version[:-2] + 'a2')
        elif r.channel == 'Beta':
            Release.objects.filter(id=r.id).update(version=r.version[:-2] + 'beta')


def get_duplicate_product_versions():
    version_ids = {}
    duplicates = {}
    for product in Release.PRODUCTS:
        version_ids[product] = {}
        for r in Release.objects.filter(product=product):
            version_ids[product].setdefault(r.version, [])
            version_ids[product][r.version].append(r.id)
            if len(version_ids[product][r.version]) > 1:
                duplicates[(product, r.version)] = version_ids[product][
                    r.version]
    return duplicates



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from datetime import datetime

from django.core.management import call_command
from django.test import TestCase
from django.test.utils import override_settings
from mock import Mock, patch
from nose.tools import eq_, ok_

from rna import admin, fields, filters, models, views


@patch('rna.management.commands.rnasync.sync_data')
@override_settings(RNA_SYNC_URL='retrovirus')
class RnaSyncCommandTests(TestCase):
    def test_call_no_args(self, sync_data_mock):
        call_command('rnasync')
        sync_data_mock.assert_called_with(url='retrovirus', clean=False,
                                          last_modified=None, api_token=None)

    def test_call_with_args(self, sync_data_mock):
        call_command('rnasync', url='megavirus', clean=True)
        sync_data_mock.assert_called_with(url='megavirus', clean=True,
                                          last_modified=None, api_token=None)


class TimeStampedModelTest(TestCase):
    @patch('rna.models.models.Model.save')
    def test_default_modified(self, mock_super_save):
        start = datetime.now()
        model = models.TimeStampedModel()
        model.save(db='test')
        ok_(model.modified > start)
        mock_super_save.assert_called_once_with(db='test')

    @patch('rna.models.models.Model.save')
    def test_unmodified(self, mock_super_save):
        model = models.TimeStampedModel()
        model.modified = space_odyssey = datetime(2001, 1, 1)
        model.save(modified=False, db='test')
        eq_(model.modified, space_odyssey)
        mock_super_save.assert_called_once_with(db='test')


class NoteTest(TestCase):
    def test_unicode(self):
        """
        Should equal note
        """
        note = models.Note(note='test')
        eq_(unicode(note), 'test')

    def test_is_known_issue_for_not_known(self):
        """
        Should be False if is_known_issue is False.
        """
        note = models.Note(is_known_issue=False)
        release = Mock(spec=models.Release())
        eq_(note.is_known_issue_for(release), False)

    def test_is_known_issue_for_wrong_release(self):
        """
        Should be True if is_known_issue is True but fixed_in_release
        doesn't match given release.
        """
        fixed_release = Mock(spec=models.Release())
        release = Mock(spec=models.Release())
        note = models.Note(is_known_issue=True,
                           fixed_in_release=fixed_release)
        eq_(note.is_known_issue_for(release), True)

    def test_is_known_issue_for(self):
        """
        Should be False if is_known_issue is True and fixed_in_release
        matches the given release.
        """
        release = Mock(spec=models.Release())
        note = models.Note(is_known_issue=True,
                           fixed_in_release=release)
        eq_(note.is_known_issue_for(release), False)


class ReleaseTest(TestCase):
    def test_unicode(self):
        """
        Should equal name
        """
        release = models.Release(product='Firefox',
                                 channel='Release',
                                 version='12.0.1')
        eq_(unicode(release), 'Firefox 12.0.1 Release')

    def test_major_version(self):
        """
        Should return the version up to, but not including, the first dot
        """
        eq_(models.Release(version='42.0').major_version(), '42')

    def test_get_bug_search_url(self):
        """
        Should return self.bug_search_url
        """
        eq_(models.Release(
            bug_search_url='http://example.com').get_bug_search_url(),
            'http://example.com')

    def test_get_bug_search_url_default(self):
        """
        Should construct based on major version
        """
        eq_(models.Release(version='42.0').get_bug_search_url(),
            'https://bugzilla.mozilla.org/buglist.cgi?'
            'j_top=OR&f1=target_milestone&o3=equals&v3=Firefox%2042&'
            'o1=equals&resolution=FIXED&o2=anyexact&query_format=advanced&'
            'f3=target_milestone&f2=cf_status_firefox42&'
            'bug_status=RESOLVED&bug_status=VERIFIED&bug_status=CLOSED&'
            'v1=mozilla42&v2=fixed%2Cverified&limit=0')

    def test_get_bug_search_url_thunderbird(self):
        """
        Should construct based on major version
        """
        eq_(models.Release(version='42.0', product='Thunderbird').get_bug_search_url(),
            'https://bugzilla.mozilla.org/buglist.cgi?'
            'classification=Client%20Software&query_format=advanced&'
            'bug_status=RESOLVED&bug_status=VERIFIED&bug_status=CLOSED&'
            'target_milestone=Thunderbird%2042.0&product=Thunderbird'
            '&resolution=FIXED'
            )

    def test_notes(self):
        """
        Should split notes into new features and known issues.
        """
        new_feature_1 = Mock(**{'is_known_issue_for.return_value': False,
                                'tag': 'Changed'})
        new_feature_2 = Mock(**{'is_known_issue_for.return_value': False})
        dot_fix = Mock(**{'is_known_issue_for.return_value': False,
                          'tag': 'Fixed',
                          'note': '42.0.1 rendering glitches'})
        known_issue_1 = Mock(**{'is_known_issue_for.return_value': True})
        known_issue_2 = Mock(**{'is_known_issue_for.return_value': True})

        with patch.object(models.Release, 'note_set') as note_set:
            release = models.Release()
            note_set.order_by.return_value = [
                new_feature_2, new_feature_1, dot_fix, known_issue_1,
                known_issue_2]
            new_features, known_issues = release.notes()
            note_set.order_by.assert_called_with('-sort_num')

        eq_(new_features, [dot_fix, new_feature_2, new_feature_1])
        eq_(known_issues, [known_issue_1, known_issue_2])

    def test_notes_public_only(self):
        """
        Should filter notes based on is_public attr.
        """
        with patch.object(models.Release, 'note_set') as note_set:
            release = models.Release()
            release.notes(public_only=True)
            note_set.order_by.return_value.filter.assert_called_with(
                is_public=True)

    @override_settings(DEV=True)
    def test_equivalent_release_for_product_dev(self):
        """
        Should return the release for the specified product with
        the same channel and major version
        """
        release = models.Release(version='42.0', channel='Release')
        release._default_manager = Mock()
        mock_order_by = release._default_manager.filter.return_value.order_by
        mock_order_by.return_value = [
            models.Release(version='42.0'), models.Release(version='42.0.1')]
        eq_(release.equivalent_release_for_product('Firefox').version,
            '42.0.1')
        release._default_manager.filter.assert_called_once_with(
            version__startswith='42.', channel='Release', product='Firefox')
        mock_order_by.assert_called_once_with('-version')

    @override_settings(DEV=False)
    def test_equivalent_release_for_product_prod(self):
        """
        Should return the release for the specified product with
        the same channel and major version, with an additional filter
        is_public=True
        """
        release = models.Release(version='42.0', channel='Release')
        release._default_manager = Mock()
        mock_order_by = release._default_manager.filter.return_value.order_by
        mock_public_filter = Mock(
            return_value=[
                models.Release(version='42.0'), models.Release(version='42.0.1')])
        mock_order_by.return_value = Mock(filter=mock_public_filter)
        eq_(release.equivalent_release_for_product('Firefox').version,
            '42.0.1')
        release._default_manager.filter.assert_called_once_with(
            version__startswith='42.', channel='Release', product='Firefox')
        mock_public_filter.assert_called_once_with(is_public=True)

    @override_settings(DEV=False)
    def test_equivalent_release_for_product_33_1(self):
        """
        Should order by 2nd version # for 33.1 after applying other
        sorting criteria
        """
        release = models.Release(version='33.1', channel='Release')
        release._default_manager = Mock()
        mock_order_by = release._default_manager.filter.return_value.order_by
        mock_public_filter = Mock(
            return_value=[
                models.Release(version='33.0.3'), models.Release(version='33.1')])
        mock_order_by.return_value = Mock(filter=mock_public_filter)
        eq_(release.equivalent_release_for_product('Firefox').version,
            '33.1')
        release._default_manager.filter.assert_called_once_with(
            version__startswith='33.', channel='Release', product='Firefox')
        mock_public_filter.assert_called_once_with(is_public=True)

    def test_no_equivalent_release_for_product(self):
        """
        Should return None for empty querysets
        """
        release = models.Release(version='42.0', channel='Release')
        release._default_manager = Mock()
        release._default_manager.filter.return_value.order_by.return_value = \
            models.Release.objects.none()
        eq_(release.equivalent_release_for_product('Firefox'), None)

    def test_equivalent_android_release(self):
        """
        Should return the equivalent_release_for_product where the
        product is 'Firefox for Android'
        """
        release = models.Release(product='Firefox')
        release.equivalent_release_for_product = Mock()
        eq_(release.equivalent_android_release(),
            release.equivalent_release_for_product.return_value)
        release.equivalent_release_for_product.assert_called_once_with(
            'Firefox for Android')

    def test_equivalent_android_release_non_firefox_product(self):
        """
        Should not call equivalent_release_for_product if
        self.product does not equal 'Firefox'
        """
        release = models.Release(product='Firefox OS')
        release.equivalent_release_for_product = Mock()
        eq_(release.equivalent_android_release(), None)
        eq_(release.equivalent_release_for_product.called, 0)

    def test_equivalent_desktop_release(self):
        """
        Should return the equivalent_release_for_product where the
        product is 'Firefox'
        """
        release = models.Release(product='Firefox for Android')
        release.equivalent_release_for_product = Mock()
        eq_(release.equivalent_desktop_release(),
            release.equivalent_release_for_product.return_value)
        release.equivalent_release_for_product.assert_called_once_with(
            'Firefox')

    def test_equivalent_desktop_release_non_firefox_product(self):
        """
        Should not call equivalent_release_for_product if
        self.product does not equal 'Firefox for Android'
        """
        release = models.Release(product='Firefox OS')
        release.equivalent_release_for_product = Mock()
        eq_(release.equivalent_desktop_release(), None)
        eq_(release.equivalent_release_for_product.called, 0)


class ISO8601DateTimeFieldTest(TestCase):
    @patch('rna.fields.parse_datetime')
    def test_strptime(self, mock_parse_datetime):
        """
        Should equal expected_date returned by mock_parse_datetime
        """
        expected_date = datetime(2001, 1, 1)
        mock_parse_datetime.return_value = expected_date
        field = fields.ISO8601DateTimeField()
        eq_(field.strptime('value', 'format'), expected_date)


class TimeStampedModelSubclass(models.TimeStampedModel):
    test = models.models.BooleanField(default=True)

    class Meta:
        app_label = 'test_rna'


class TimestampedFilterBackendTest(TestCase):
    @patch('rna.filters.DjangoFilterBackend.get_filter_class',
           return_value='The Dude')
    def test_filter_class(self, mock_super_get_filter_class):
        """
        Should return super call if view has filter_class attr
        """
        mock_view = Mock(filter_class='abides')
        filter_backend = filters.TimestampedFilterBackend()
        eq_(filter_backend.get_filter_class(mock_view), 'The Dude')

    @patch('rna.filters.DjangoFilterBackend.get_filter_class',
           return_value='The Dude')
    def test_filter_fields(self, mock_super_get_filter_class):
        """
        Should return super call if view has filter_fields attr
        """
        mock_view = Mock(filter_fields='abides')
        filter_backend = filters.TimestampedFilterBackend()
        eq_(filter_backend.get_filter_class(mock_view), 'The Dude')

    @patch('rna.filters.DjangoFilterBackend.get_filter_class')
    def test_no_queryset(self, mock_super_get_filter_class):
        """
        Should return None if queryset is None (the default)
        """
        view = 'nice'
        filter_backend = filters.TimestampedFilterBackend()
        eq_(filter_backend.get_filter_class(view), None)
        ok_(not mock_super_get_filter_class.called)

    @patch('rna.filters.DjangoFilterBackend.get_filter_class')
    def test_non_timestampedmodel(self, mock_super_get_filter_class):
        """
        Should return None if queryset.model is not a subclass of
        models.TimeStampedModel
        """
        view = 'nice'
        queryset = Mock(model=models.models.Model)  # model
        filter_backend = filters.TimestampedFilterBackend()
        eq_(filter_backend.get_filter_class(view, queryset=queryset), None)
        ok_(not mock_super_get_filter_class.called)

    @patch('rna.filters.DjangoFilterBackend.get_filter_class')
    def test_default(self, mock_super_get_filter_class):
        """
        Should return a subclass of the default_filter_set instance
        attr with the inner Meta class model attr equal to the queryset
        model and fields equal to all of the model fields except
        created and modified, and in addition the created_before,
        created_after, modified_before, and modified_after fields
        """
        view = 'nice'
        queryset = Mock(model=TimeStampedModelSubclass)
        filter_backend = filters.TimestampedFilterBackend()
        filter_class = filter_backend.get_filter_class(view, queryset=queryset)
        eq_(filter_class.Meta.model, TimeStampedModelSubclass)
        eq_(filter_class.Meta.fields,
            ['created_before', 'created_after', 'modified_before',
             'modified_after', 'id', 'test'])
        ok_(not mock_super_get_filter_class.called)

    @patch('rna.filters.DjangoFilterBackend.get_filter_class')
    def test_exclude_fields(self, mock_super_get_filter_class):
        """
        Should not include fields named in the view.
        """
        mock_view = Mock(
            filter_class=None, filter_fields=None,
            filter_fields_exclude=('created_before', 'id'))
        queryset = Mock(model=TimeStampedModelSubclass)
        filter_backend = filters.TimestampedFilterBackend()
        filter_class = filter_backend.get_filter_class(
            mock_view, queryset=queryset)
        eq_(filter_class.Meta.model, TimeStampedModelSubclass)
        eq_(filter_class.Meta.fields,
            ['created_after', 'modified_before', 'modified_after', 'test'])
        eq_(mock_super_get_filter_class.called, 0)


class URLsTest(TestCase):
    @patch('rest_framework.routers.DefaultRouter.register')
    @patch('rest_framework.routers.DefaultRouter.urls')
    def test_urls(self, mock_urls, mock_register):
        from . import urls
        mock_register.assert_any_call('notes', views.NoteViewSet)
        mock_register.assert_any_call('releases', views.ReleaseViewSet)
        ok_(set(mock_urls).issubset(urls.urlpatterns))


class ReleaseAdminTest(TestCase):
    @patch('rna.admin.ReleaseAdmin.message_user')
    @patch('rna.admin.now')
    def test_copy_releases(self, mock_now, mock_message_user):
        mock_release_model = Mock()
        mock_release_model.objects.filter.return_value.count.return_value = 1
        release_admin = admin.ReleaseAdmin(mock_release_model, 'admin_site')
        mock_release = Mock(id=1, version='42.0', product='Firefox')
        mock_release.note_set.all.return_value = ['note']

        release_admin.copy_releases('request', [mock_release])

        mock_release_model.objects.filter.assert_called_once_with(
            version__endswith='42.0', product='Firefox')
        eq_(mock_release.id, None)
        eq_(mock_release.version, 'copy-42.0')
        mock_release.save.assert_called_once_with()
        mock_release.note_set.add.assert_called_once_with('note')
        mock_release.note_set.update.assert_called_once_with(
            modified=mock_now.return_value)
        mock_message_user.assert_called_once_with('request', 'Copied Release')

    @patch('rna.admin.ReleaseAdmin.message_user')
    @patch('rna.admin.now')
    def test_2nd_copy_releases(self, mock_now, mock_message_user):
        mock_release_model = Mock()
        mock_release_model.objects.filter.return_value.count.return_value = 2
        release_admin = admin.ReleaseAdmin(mock_release_model, 'admin_site')
        mock_release = Mock(id=1, version='42.0', product='Firefox')
        mock_release.note_set.all.return_value = ['note']

        release_admin.copy_releases('request', [mock_release])

        mock_release_model.objects.filter.assert_called_once_with(
            version__endswith='42.0', product='Firefox')
        eq_(mock_release.id, None)
        eq_(mock_release.version, 'copy2-42.0')
        mock_release.save.assert_called_once_with()
        mock_release.note_set.add.assert_called_once_with('note')
        mock_release.note_set.update.assert_called_once_with(
            modified=mock_now.return_value)
        mock_message_user.assert_called_once_with('request', 'Copied Release')



DEBUG = True
TEMPLATE_DEBUG = DEBUG

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

USE_TZ = False
SITE_ID = 1
SECRET_KEY = 'gxxg@@juj%4=-jr5ohv3cdj6)v6p2j5e3q91naw#m&amp;&amp;dgzq-zh'

ROOT_URLCONF = 'tests.urls'
STATIC_URL = '/static/'

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django_extensions',
    'django_nose',
    'pagedown',
    'rest_framework',
    'rest_framework.authtoken',
    'rna',
)

TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'
NOSE_ARGS = ('--nocapture', )

REST_FRAMEWORK = {
    # Use hyperlinked styles by default.
    # Only used if the `serializer_class` attribute is not set on a view.
    'DEFAULT_MODEL_SERIALIZER_CLASS': (
        'rna.serializers.HyperlinkedModelSerializerWithPkField'),

    # Use Django's standard `django.contrib.auth` permissions,
    # or allow read-only access for unauthenticated users.
    'DEFAULT_PERMISSION_CLASSES': (
        'rest_framework.permissions.DjangoModelPermissionsOrAnonReadOnly',
    ),

    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.TokenAuthentication',
    ),

    'DEFAULT_FILTER_BACKENDS': ('rna.filters.TimestampedFilterBackend',)
}

RNA = {'BASE_URL': 'https://nucleus.mozilla.org/rna/'}



from django import forms
from django.utils.dateparse import parse_datetime


class ISO8601DateTimeField(forms.DateTimeField):
    def strptime(self, value, format):
        return parse_datetime(value)



from django.conf.urls import patterns, include, url
from django.contrib.staticfiles.urls import staticfiles_urlpatterns

from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns(
    '',
    url(r'^', include('rna.urls')),
    url(r'^admin/', include(admin.site.urls)),
)
urlpatterns += staticfiles_urlpatterns()



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from setuptools import setup

setup(
    name='django-mozilla-rna',
    version='2.0.2',
    description='Django app for managing Mozilla product release notes.',
    author='Josh Mize',
    author_email='jmize@mozilla.com',
    url='https://github.com/mozilla/rna/',
    license='MPL 2.0',
    packages=['rna', 'rna.management', 'rna.management.commands'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Django>=1.8,<1.9',
        'djangorestframework>=3.3.0',
        'django-extensions>=1.2.0',
        'django-synctool',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
    ],
)



from django.conf import settings
from django.core.management import BaseCommand

from synctool.functions import sync_data

from rna.utils import get_last_modified_date


DEFAULT_RNA_SYNC_URL = 'https://nucleus.mozilla.org/rna/sync/'


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('-u', '--url',
                            default=getattr(settings, 'RNA_SYNC_URL', DEFAULT_RNA_SYNC_URL),
                            help='Full URL to RNA Sync endpoint')
        parser.add_argument('-c', '--clean', action='store_true',
                            help='Delete all RNA data before sync.')

    def handle(self, *args, **options):
        clean = options['clean']
        sync_data(url=options['url'],
                  clean=clean,
                  last_modified=None if clean else get_last_modified_date(),
                  api_token=None)
